/*
 * Copyright (c) 2000 S.Eguchi(fukurou@kissa.ne.jp)
 * All rights reserved.
 */
#include <stdio.h>
#include <errno.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "setihodai.h"
#include "GetWUDirStatus.h"
#include "CheckMakeWUDir.h"
#include "Log.h"

#define SH_MAIN 
#include "GblArea.h"


int	doLoop;


int main(int argc, char* argv[]) 
{

	char			dirname[1024];
	char			setipath[1024];
	char			cwd[1024];
	int				i;
	int				pid;
	int				wpid;
	int				wstatus;
	int				execret;
	WUSTAT*			wuStatTop;
	WUSTAT*			wuStatP;
	WUSTAT*			wuStatExec;
	int				execIndex;
	time_t			firstWUTime;



	sprintf(g_msgWork, "Start setihodai pid=%d",getpid());
	LOGOUT(g_msgWork);

	/* read configuration from file */
	if(ConfInit() != 0) {
		LOGOUT("Invalid configuration!! exit.");
		return(1);
	}

	/* Initialize Global Area */
	InitGlobal();

	/* allocate working unit table */
	wuStatTop = (WUSTAT*)malloc(sizeof(WUSTAT)*g_wuNum);
	if(wuStatTop == NULL) {
		sprintf(g_msgWork, "Memory full!!", strerror(errno));
		LOGOUT(g_msgWork);
		return(1);
	}
	bzero(wuStatTop, sizeof(WUSTAT)*g_wuNum);


	if(getcwd(cwd, sizeof(cwd)) == 0) {
		sprintf(g_msgWork, "Process execution error(%s)", strerror(errno));
		LOGOUT(g_msgWork);
		return(1);
	}

	if(CheckMakeWUDir(".", g_wuNum)!=0) {
		LOGOUT("Check & Make WU working dir error");
		return(1);
	}

	doLoop=1;
	while(doLoop) {
		/* get work unit directory status */
		if(GetAllWUStat(wuStatTop, g_wuNum, ".") != 0) {
			LOGOUT("Invalid Working Unit directory ");
			return(1);
		}

		/* search oldest WU file directory */
		execIndex=(-1);
		firstWUTime=0x7FFFFFFF;
		wuStatP = wuStatTop;
		wuStatExec=NULL;
		for(i=0;i<g_wuNum;i++) {
			printf("i=%d stat=%d modtime=%d\n", 
					i,
					wuStatP->dirStat,
					wuStatP->wuModTime);
			if( (firstWUTime > wuStatP->wuModTime) 
				&& (wuStatP->wuModTime!=0) ) {
				firstWUTime = wuStatP->wuModTime;
				execIndex=i;
				wuStatExec=wuStatP;
			}
			wuStatP++;
		}
		if(execIndex == (-1)) {
			LOGOUT("Can not found unfinsed WorkUnit(s)");
			LOGOUT("Sleep 300secs");
			sleep(300);
			continue;
		}

		/* execute setiathome client */
		pid=fork();
		if(pid < 0) {
			LOGOUT("Can not fork child process");
			LOGOUT("Sleep 300secs");
			sleep(300);
			continue;
		}
		else if(pid == 0) {
			/* child process */
			sprintf(g_msgWork, "Starting setiathome. WU number %d pid=%d", 
									execIndex, getpid());
			LOGOUT(g_msgWork);
			sprintf(setipath, "%s/%s", cwd, SETI_MODULE_NAME);

			sprintf(g_msgWork, "SETI PATH = %s ",setipath);
			DBGOUT(g_msgWork);

			sprintf(dirname, "%s/%s%03d", ".", WU_DIR_SUF, execIndex);
			DBGOUT(dirname);

			if(chdir(dirname) != 0) {
				sprintf(g_msgWork, "working directory error(%s)", strerror(errno));
				LOGOUT(g_msgWork);
				exit(1);
			}
			g_execArg[g_execArgNum] = SETP_STOP_AFT_PROC;
			g_execArg[g_execArgNum+1] = NULL;
			execret = execv(setipath, g_execArg);
			exit(execret);
		}
		/* parent process */
		wuStatExec->setiPid = pid;
		wpid = wait(&wstatus);
		if(WEXITSTATUS(wstatus) != 0) {
			sprintf(g_msgWork, "setiathome execution error rc=%d ", WEXITSTATUS(wstatus));
			LOGOUT(g_msgWork);
			LOGOUT("Sleep 300secs");
			sleep(300);
		}
		wuStatExec->setiPid = 0;
	}


	return(0);
}

int GetAllWUStat(WUSTAT* wuStatTop, int wuNum, char* dir)
{
	int		i;
	char	dirname[1024];
	WUSTAT*	wup;


	wup = wuStatTop;

	for(i=0;i<wuNum;i++) {
		sprintf(dirname, "%s/%s%03d", dir, WU_DIR_SUF, i);

		wup->dirStat = GetWUDirStat(dirname);

		if(wup->dirStat == WU_DIR_STAT_EXECUTION) {
			wup->wuModTime = GetWUModifyTime(dirname);
		}
		else {
			wup->wuModTime = 0;
		}
		wup++;
	}
	return(0);
}
